(function() {
    'use strict';
    
    var gulp        = require('gulp'),
        jshint      = require('gulp-jshint'),
        
        Util        = require('util-io'),
        fs          = require('fs'),
        test        = require('./test/test'),
        exec        = require('child_process').exec,
        Info        = require('./package'),
        
        LIB         = './lib/',
        Src         = [
            './*.js',
            'test/**/*.js',
            LIB + '/**/*.js'
        ];
    
    gulp.task('jshint', function() {
        gulp.src(Src)
            .pipe(jshint())
            .pipe(jshint.reporter())
            .on('error', onError);
    });
    
    gulp.task('test', function() {
       test.uglify();
    });
    
     gulp.task('changelog', function() {
        var version     = 'v' + Info.version,
            name        = 'ChangeLog',
            
            gitTempl    = 'git log {{ version }}..HEAD --pretty=format:"- %s" --grep {{ category }} | sed  \'s/{{ category }}//g\'',
            
            gitFix      = Util.render(gitTempl, {
               category : 'fix' ,
               version  : version
            }),
            
            gitFeature  = Util.render(gitTempl, {
               category: 'feature',
               version  : version
            }),
            
            versionNew  = getNewVersion();
        
        if (versionNew)
            versionNew  = 'v' + versionNew;
        else
            versionNew  = version + '?';
        
        Util.exec.parallel([
            Util.exec.with(fs.readFile, name, 'utf8'),
            Util.exec.with(exec, gitFix),
            Util.exec.with(exec, gitFeature),
            ], function(error, fileData, fixData, featureData) {
                var DATA        = 0,
                    STD_ERR     = 1,
                    fix         = fixData[DATA],
                    feature     = featureData[DATA],
                    date        = Util.getShortDate(),
                    head        = date + ', ' + versionNew + '\n\n',
                    data        = '';
                
                console.log(fix || feature);
                
                if (fix || feature) {
                    data        = head;
                    
                    if (fix) {
                        data    += 'fix:'       + '\n';
                        data    += fix          + '\n\n';
                    }
                    
                    if (feature) {
                        data    += 'feature:'   + '\n';
                        data    += feature      + '\n\n';
                    }
                    
                    data        += '\n';
                    data        += fileData;
                }
                
                error   = error || fixData[STD_ERR] || featureData[STD_ERR];
                
                if (error)
                    console.log(error);
                else if (!data)
                    console.log('No new feature and fix commits from v', version);
                else
                    fs.writeFile(name, data, function(error) {
                        var msg = 'changelog: done';
                        
                        console.log(error || msg);
                    });
            });
    });
    
     gulp.task('package', function() {
        var data,
            msg         = 'ERROR: version is missing. gulp package --v<version>',
            version     = Info.version,
            versionNew  = getNewVersion();
        
        if (!versionNew) {
            console.log(msg);
        } else {
            Info.version    = versionNew;
            data            = JSON.stringify(Info, 0, 2) + '\n';
            Info.version    = version;
            
            fs.writeFile('package.json', data, function(error) {
                var msg = 'package: done';
                
                console.log(error || msg);
            });
        }
    });
    
    gulp.task('docs', function() {
        var version     = Info.version,
            versionNew  = getNewVersion(),
            msg         = 'ERROR: version is missing. gulp docs --v<version>';
        
        if (!versionNew) {
            console.log(msg);
        } else {
            replaceVersion('README.md', version, versionNew);
            replaceVersion('HELP.md', version, versionNew);
        }
    });
    
    gulp.task('release', ['changelog', 'docs', 'package']);
    gulp.task('default', ['jshint', 'test']);
    
    function onError(params) {
        console.log(params.message);
    }
    
    function getNewVersion() {
        var versionNew,
            argv        = process.argv,
            length      = argv.length - 1,
            last        = process.argv[length],
            regExp      = new RegExp('^--'),
            isMatch     = last.match(regExp);
            
        if (isMatch)
            versionNew  = last.substr(3);
        
        return versionNew;
    }
    
    function replaceVersion(name, version, versionNew, callback) {
         fs.readFile(name, 'utf8', function(error, data) {
                if (error) {
                    console.log(error);
                } else {
                    data = data.replace(version, versionNew);
                    
                    fs.writeFile(name, data, function(error) {
                        var msg = 'done: ' + name;
                        
                        console.log(error || msg);
                        Util.exec(callback);
                    });
                }
            });
    }
    
})();
